--define global vars and defaults (saved between sessions)
local function SetDefaults()
	KChefTime_Settings = {
		["Auto"] = true,
		["Self"] = 2,
		["Party"] = 0,
		["Raid"] = 0,
	}
end
SetDefaults()

function KChefTimeFrame_OnEvent(self,event,...)
	if event == "READY_CHECK" and KChefTime_Settings.Auto then
		KChefTime_Command()
	end
	if event == "ADDON_LOADED" and select(1,...) == "KChefTime" then
		--vars loaded from saved settings
		KChefTime_LoadOptionSettings() --load global vars to option controls
	end
end

function KChefTimeFrame_OnLoad(self)
 	SlashCmdList["KChefTime"]=KChefTime_Command
	SLASH_KChefTime1, SLASH_KChefTime2="/KChef","/Chef"
	KChefTimeFrame:RegisterEvent("READY_CHECK")
	KChefTimeFrame:RegisterEvent("ADDON_LOADED")
	print("Loaded.")
	KChefTime_Command("help")
end

function KChefTime_Command(Msg,Editbox)
	if Msg then Msg=strlower(Msg); end
	if Msg == "help" then
		print("Use /KChef or /Chef.")
	else
		local RaidCount = GetNumRaidMembers()
		local PartyCount = GetNumPartyMembers()
		local GroupIndex, GroupCount, Unit, BuffIndex
		local ToonName, BuffName, ExpirationTime
		local Toons={} --Name, WellFedMins, ElixirName, ElixirMins
		local OutputSumFed, OutputSumElixir, OutputDetails, OutputDetailsMsg, OutputFooter
		local FedCount = 0
		local ElixiredCount = 0
		local Index

		--how many toons?
		if RaidCount > 0 then
			GroupCount = RaidCount
			Unit = "raid"
		elseif PartyCount > 0 then
			GroupCount = PartyCount
			Unit = "party"
		else --player or target unit
			GroupCount = 1
			if UnitExists("target") then
				Unit = "target"
			else
				Unit = "player"
			end
		end
		
		--gather data
		for GroupIndex = 1, (GroupCount) do
			if GroupCount == 1 then
				ToonName = GetUnitName(Unit)
			else
				ToonName = GetUnitName(Unit..GroupIndex)
			end
			Toons[ToonName]={WellFed=0,ElixirName="",ElixirExpiration=0}
			BuffIndex = 1
			if GroupCount == 1 then
				BuffName, _, _, _, _, _, ExpirationTime, _, _, _, _ = UnitBuff(Unit, BuffIndex)
			else
				BuffName, _, _, _, _, _, ExpirationTime, _, _, _, _ = UnitBuff(Unit..GroupIndex, BuffIndex)
			end
			while BuffName do
				if BuffName == "Well Fed" then
					Toons[ToonName].WellFed = floor(((ExpirationTime-GetTime())/60)+1)
				elseif strfind(strlower(BuffName), "elixir") then
					Toons[ToonName].ElixirName = BuffName
					Toons[ToonName].ElixirExpiration = floor(((ExpirationTime-GetTime())/60)+1)
				elseif strfind(strlower(BuffName), "flask") then
					Toons[ToonName].ElixirName = BuffName
					Toons[ToonName].ElixirExpiration = floor(((ExpirationTime-GetTime())/60)+1)
				end
				BuffIndex = BuffIndex + 1
				if GroupCount == 1 then
					BuffName, _, _, _, _, _, ExpirationTime, _, _, _, _ = UnitBuff(Unit, BuffIndex)
				else
					BuffName, _, _, _, _, _, ExpirationTime, _, _, _, _ = UnitBuff(Unit..GroupIndex, BuffIndex)
				end
			end
		end
		
		--build print results
		OutputSumFed = ""
		OutputSumElixir = ""
		OutputDetails = {}
		OutputDetailsMsg = ""
		for GroupIndex = 1, (GroupCount) do
			if GroupCount == 1 then
				ToonName = GetUnitName(Unit)
			else
				ToonName = GetUnitName(Unit..GroupIndex)
			end
			if Toons[ToonName].WellFed > 0 then
				if Toons[ToonName].WellFed > 0 then
					OutputDetailsMsg = ToonName..": Well fed ("..Toons[ToonName].WellFed..")"
				else
					OutputDetailsMsg = ToonName..": Well fed"
				end
				FedCount = FedCount + 1
			else
				if OutputSumFed ~= "" then OutputSumFed = OutputSumFed..", "; end
				OutputSumFed = OutputSumFed..ToonName
				OutputDetailsMsg = ToonName..": Not well fed"
			end
			if Toons[ToonName].ElixirName ~= "" then
				if Toons[ToonName].ElixirExpiration > 0 then
					OutputDetailsMsg = OutputDetailsMsg.." Elixired/flasked with "..Toons[ToonName].ElixirName.." ("..Toons[ToonName].ElixirExpiration..")."
				else
					OutputDetailsMsg = OutputDetailsMsg.." Elixired/flasked with "..Toons[ToonName].ElixirName.."."
				end
				ElixiredCount = ElixiredCount + 1
			else
				if OutputSumElixir ~= "" then OutputSumElixir = OutputSumElixir..", "; end
				OutputSumElixir = OutputSumElixir..ToonName
				OutputDetailsMsg = OutputDetailsMsg..": not elixired/flasked."
			end
			table.insert(OutputDetails, OutputDetailsMsg)
		end
		OutputFooter = "Of "..GroupCount..", there are "..FedCount.." well fed & "..ElixiredCount.." are elixired/flasked."

		--print header
		print("Chef Time:") --header
		--print summary results to self?
		if KChefTime_Settings.Self == 1 then
			if OutputSumFed ~= "" then
				print("Not well fed: "..OutputSumFed)
			else
				print("Everyone is well fed")
			end
			if OutputSumElixir ~= "" then
				print("Not elixired/flasked: "..OutputSumElixir)
			else
				print("Everyone is elixired/flasked")
			end
			if GroupCount > 1 then print(OutputFooter); end
		end
		--print summary results to group?
		if Unit == "party" and KChefTime_Settings.Party == 1 or 
			Unit == "raid" and KChefTime_Settings.Raid == 1 then
			if OutputSumFed ~= "" then
				SendChatMessage("Not well fed: "..OutputSumFed, strupper(Unit))
			else
				SendChatMessage("Everyone is well fed", strupper(Unit))
			end
			if OutputSumElixir ~= "" then
				SendChatMessage("Not elixired/flasked: "..OutputSumElixir, strupper(Unit))
			else
				SendChatMessage("Everyone is elixired/flasked", strupper(Unit))
			end
			SendChatMessage(OutputFooter, strupper(Unit))
		end
		--print detailed results to self?
		if KChefTime_Settings.Self == 2 then
			for Index=1, #OutputDetails, 1 do 
				print(Index..": "..OutputDetails[Index])
			end
			if GroupCount > 1 then print(OutputFooter); end
		end
		--print detailed results to group?
		if Unit == "party" and KChefTime_Settings.Party == 2 or 
			Unit == "raid" and KChefTime_Settings.Raid == 2 then
			for Index=1, #OutputDetails, 1 do 
				SendChatMessage(Index..": "..OutputDetails[Index], strupper(Unit))
			end
			SendChatMessage(Index..": "..OutputFooter, strupper(Unit))
		end
	end
end


function KChefTimeOptionsFrame_OnLoad(panel)
	local WoWver = select(4, GetBuildInfo())
	-- Set the name for the Category for the Panel
	panel.name = "Chef Time v" .. GetAddOnMetadata("KChefTime", "Version")
	--KChefTimeOptionsFrame_Title:SetText("Chef Time v" .. GetAddOnMetadata("KChefTime", "Version"))

	--define the functions that will run when the user clicks on the associated buttons
	panel.okay = function(self) KChefTimeOptionsFrame_Close(); end
	panel.cancel = function(self)  KChefTimeOptionsFrame_Cancel(); end
	panel.default = function(self)  KChefTimeOptionsFrame_Defaults(); end
	--panel.parent = ""
	--panel.refresh = function(self)  KChefTimeOptions_Refresh();  end

	-- Add the panel to the Interface Options
	InterfaceOptions_AddCategory(panel)

	--define the direct to options command
	--SlashCmdList["KChefTimeOptions"] = KChefTimeOptions_Command
	--SLASH_KChefTimeOptions1,SLASH_KChefTimeOptions2 = "/klm","/LifeMon"
end

function KChefTimeOptionsFrame_Close()
	--save option control settings to global vars
	KChefTime_Settings.Self=KChefTimeOptionsFrame_CBAuto:GetChecked()
	if KChefTimeOptionsFrame_RBSelfNone:GetChecked() then
		KChefTime_Settings.Self=0
	elseif KChefTimeOptionsFrame_RBSelfSummary:GetChecked() then
		KChefTime_Settings.Self=1
	elseif KChefTimeOptionsFrame_RBSelfDetails:GetChecked() then
		KChefTime_Settings.Self=2
	end
	if KChefTimeOptionsFrame_RBPartyNone:GetChecked() then
		KChefTime_Settings.Party=0
	elseif KChefTimeOptionsFrame_RBPartySummary:GetChecked() then
		KChefTime_Settings.Party=1
	elseif KChefTimeOptionsFrame_RBPartyDetails:GetChecked() then
		KChefTime_Settings.Party=2
	end
	if KChefTimeOptionsFrame_RBRaidNone:GetChecked() then
		KChefTime_Settings.Raid=0
	elseif KChefTimeOptionsFrame_RBRaidSummary:GetChecked() then
		KChefTime_Settings.Raid=1
	elseif KChefTimeOptionsFrame_RBRaidDetails:GetChecked() then
		KChefTime_Settings.Raid=2
	end
end

function KChefTimeOptionsFrame_Cancel()
	KChefTime_LoadOptionSettings()
end

function KChefTimeOptionsFrame_Defaults()
	SetDefaults()
	KChefTime_LoadOptionSettings()
end

function KChefTime_LoadOptionSettings(...)
	local DoThis
	if select(1,...) ~= nil then
		DoThis = select(1,...)
	else
		if KChefTime_Settings.Self == 0 then KChefTime_LoadOptionSettings("SN")
		elseif KChefTime_Settings.Self == 1 then KChefTime_LoadOptionSettings("SS")
		elseif KChefTime_Settings.Self == 2 then KChefTime_LoadOptionSettings("SD")
		end
		if KChefTime_Settings.Party == 0 then KChefTime_LoadOptionSettings("PN")
		elseif KChefTime_Settings.Party == 1 then KChefTime_LoadOptionSettings("PS")
		elseif KChefTime_Settings.Party == 2 then KChefTime_LoadOptionSettings("PD")
		end
		if KChefTime_Settings.Raid == 0 then KChefTime_LoadOptionSettings("RN")
		elseif KChefTime_Settings.Raid == 1 then KChefTime_LoadOptionSettings("RS")
		elseif KChefTime_Settings.Raid == 2 then KChefTime_LoadOptionSettings("RD")
		end
	end
	--set option controls from global vars or when user changes selection in options
	KChefTimeOptionsFrame_CBAuto:SetChecked(KChefTime_Settings.Auto)
	if DoThis == "SN" then
		KChefTimeOptionsFrame_RBSelfNone:SetChecked(true)
		KChefTimeOptionsFrame_RBSelfSummary:SetChecked(false)
		KChefTimeOptionsFrame_RBSelfDetails:SetChecked(false)
	elseif DoThis == "SS" then
		KChefTimeOptionsFrame_RBSelfNone:SetChecked(false)
		KChefTimeOptionsFrame_RBSelfSummary:SetChecked(true)
		KChefTimeOptionsFrame_RBSelfDetails:SetChecked(false)
	elseif DoThis == "SD" then
		KChefTimeOptionsFrame_RBSelfNone:SetChecked(false)
		KChefTimeOptionsFrame_RBSelfSummary:SetChecked(false)
		KChefTimeOptionsFrame_RBSelfDetails:SetChecked(true)
	end
	if DoThis == "PN" then
		KChefTimeOptionsFrame_RBPartyNone:SetChecked(true)
		KChefTimeOptionsFrame_RBPartySummary:SetChecked(false)
		KChefTimeOptionsFrame_RBPartyDetails:SetChecked(false)
	elseif DoThis == "PS" then
		KChefTimeOptionsFrame_RBPartyNone:SetChecked(false)
		KChefTimeOptionsFrame_RBPartySummary:SetChecked(true)
		KChefTimeOptionsFrame_RBPartyDetails:SetChecked(false)
	elseif DoThis == "PD" then
		KChefTimeOptionsFrame_RBPartyNone:SetChecked(false)
		KChefTimeOptionsFrame_RBPartySummary:SetChecked(false)
		KChefTimeOptionsFrame_RBPartyDetails:SetChecked(true)
	end
	if DoThis == "RN" then
		KChefTimeOptionsFrame_RBRaidNone:SetChecked(true)
		KChefTimeOptionsFrame_RBRaidSummary:SetChecked(false)
		KChefTimeOptionsFrame_RBRaidDetails:SetChecked(false)
	elseif DoThis == "RS" then
		KChefTimeOptionsFrame_RBRaidNone:SetChecked(false)
		KChefTimeOptionsFrame_RBRaidSummary:SetChecked(true)
		KChefTimeOptionsFrame_RBRaidDetails:SetChecked(false)
	elseif DoThis == "RD" then
		KChefTimeOptionsFrame_RBRaidNone:SetChecked(false)
		KChefTimeOptionsFrame_RBRaidSummary:SetChecked(false)
		KChefTimeOptionsFrame_RBRaidDetails:SetChecked(true)
	end
end

function KChefTimeRB_OnClick(...)
	--toggle option controls when one is selected
	KChefTime_LoadOptionSettings(...)
end